package com.afp.ordermanagement.controller;

import com.afp.ordermanagement.model.Order;
import com.afp.ordermanagement.reactivekafkaservice.Receiver;
import com.afp.ordermanagement.reactivekafkaservice.Sender;
import com.afp.ordermanagement.service.OrderService;
import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import static java.util.Objects.isNull;

@RestController
@RequestMapping("/api")
@CrossOrigin
public class OrderController {

    @Autowired
    private OrderService orderService;

    @Autowired
    Sender sender;

    @Autowired
    Receiver receiver;


    /**
     * DESC -   Persisting order information in the database and sending the
     *          order object via orders kafka topic so that warehouse can
     *          handle that
     * @param orderObject
     * @return
     */
    @PostMapping("/orders")
    @ResponseStatus(HttpStatus.CREATED)
    public Mono<Order> getOrderFromEcom(@RequestBody Order orderObject) {
        sender.sendOrderToWarehouse(orderObject);
        return orderService.createOrder(orderObject);
    }

    @GetMapping("/orders")
    @CrossOrigin
    public Flux<Order> getAllOrders() {
        return orderService.getAllOrders();
    }

    @PostMapping("/orders/filtered")
    @CrossOrigin
    public Flux<Order> getFilteredOrders(@RequestBody AllOrdersRequestBody reqBody) {
        return orderService.getQueriedOrders(reqBody);
    }

    @GetMapping("/orders/{orderId}")
    @CrossOrigin
    public Mono<Order> getOrderById(@PathVariable("orderId") String orderId) {
        return orderService.getOrderById(orderId);
    }

    @GetMapping("/orders/customer/{customerId}")
    public Flux<Order> getAllOrdersByCustomerId(@PathVariable("customerId") String customerId) {
        return orderService.getAllOrdersByCustomerId(customerId);
    }
    
    @PutMapping("/order/{orderId}")
    public Mono<ResponseEntity<Order>> updateOrder(@PathVariable(value = "orderId") String orderId, @RequestBody Order order){
        return orderService.updateOrderByOrderId(orderId, order)
                .map(updatedOrder -> ResponseEntity.ok(updatedOrder))
                .defaultIfEmpty(ResponseEntity.notFound().build());
    }

    @DeleteMapping("/orders/{orderId}")
    public void deleteOrderbyId(@PathVariable(value = "orderId") String orderId) {
        orderService.deleteOrderById(orderId);
    }
}
